<?php
/**
 * @file
 * WatchdogSearch class.
 * This file contains implementation of all functions needed for watchdog
 * log search.
 */

require_once 'LogmanSearchInterface.php';

/**
 * Implements the search, filtering and statistics
 * function for drupal watchdog logs.
 */
class LogmanDblogSearch implements LogmanSearchInterface {
  protected $searchKey;
  protected $type;
  protected $limit;
  protected $quantity = 5;

  /**
   * Constructor for initializing the class variables.
   *
   * @param null $search_key
   *   Value to search.
   * @param null $type
   *   Log type to search.
   * @param int $limit
   *   Number of log items on a page.
   */
  public function __construct($search_key = NULL, $type = NULL, $limit = 5) {
    $this->searchKey = $search_key;
    $this->type = $type;
    $this->limit = $limit;
  }

  /**
   * A function to get the statistics based on dblog.
   *
   * @param string $url
   *   Page URL.
   * @param string $against
   *   Field value on which statistics data to be grouped.
   * @param array $date_range
   *   The date range value on which data will be fetched.
   *
   * @return array
   *   An array of statistics data.
   */
  public function getStatistics($url = '', $against = 'severity', $date_range = array()) {
    // Basic sql query to fetch statistics.
    $sql  = "SELECT %s, COUNT(*) as 'count' FROM {watchdog}";
    $query_tok_values = array();
    array_push($query_tok_values, $against);

    $sql_criteria = '';

    // Filter on url if it is mentioned.
    if (!empty($url)) {
      $sql_criteria .= " location = '$url' ";
    }

    // Add the date range if it is mentioned.
    if (!empty($date_range)) {
      $sql_criteria .= ' AND timestamp >= ' . implode(' AND timestamp <= ', $date_range);
    }

    // Add the criteria to the sql query.
    if (!empty($sql_criteria)) {
      $sql .= ' WHERE ' . $sql_criteria;
    }

    // Add the grouping and sorting.
    $sql .= ' GROUP BY %s ORDER BY %s ';
    array_push($query_tok_values, $against);
    array_push($query_tok_values, $against);

    // Prepare statistics array.
    $result = db_query($sql, $query_tok_values);
    $statistics = array();
    while ($row = db_fetch_object($result)) {
      $statistics[] = $row;
    }
    return $statistics;
  }

  /**
   * A function to get the url wise page statistics based on the watchdog log.
   *
   * @param string $url
   *   Page URL.
   * @param array $date_range
   *   The date range value on which data will be fetched.
   *
   * @return array
   *   An array of statistics data.
   */
  public function getPageStatistics($url, $date_range = array()) {
    return $this->getStatistics($url, 'severity', $date_range);
  }

  /**
   * Function to get the log search query count.
   *
   * @param string $sql
   *   SQL query string.
   * @param array $arguments
   *   Query arguments array.
   *
   * @return int
   *   Count of items.
   */
  protected function searchCountQuery($sql, $arguments = array()) {
    $sql = str_replace('*', 'COUNT(*)', $sql);
    return db_result(db_query($sql, $arguments));
  }

  /**
   * Function to set limit.
   *
   * @param int $limit
   *   Number of log items on a page.
   */
  public function setLimit($limit) {
    $this->limit = $limit;
  }

  /**
   * Function to set quantity.
   *
   * @param int $quantity
   *   Number of pagination items on a page.
   */
  public function setQuantity($quantity) {
    $this->quantity = $quantity;
  }

  /**
   * A function to search log on all fields of watchdog.
   *
   * @param array $params
   *   Additional log fields to search.
   *
   * @return object
   *   Object containing filtered log data.
   */
  public function searchLog($params = array()) {
    // Basic sql search query with place holders.
    $sql = "SELECT * FROM {watchdog}";
    $query_tok_values = array();

    // Apply the search criteria.
    // Search key.
    $sql_criteria = array();
    if (!empty($this->searchKey)) {
      $sql_criteria[] = "(message like '%%%s%' OR variables like '%%%s%')";
      array_push($query_tok_values, $this->searchKey);
      array_push($query_tok_values, $this->searchKey);
    }

    // Type.
    if (!empty($this->type)) {
      $sql_criteria[] = "type = '%s'";
      array_push($query_tok_values, $this->type);
    }
    // All other params.
    if (is_array($params) && !empty($params)) {
      // Use date range and remove form array.
      // So that it doesn't gets processed later.
      // The date range is expected to be an array.
      $date_range = $params['date_range'];
      unset($params['date_range']);
      if (!empty($date_range)) {
        $sql_criteria[] = "timestamp >= " . implode(" AND timestamp <= ", $date_range);
      }
      foreach ($params as $key => $value) {
        if (!empty($value)) {
          $sql_criteria[] = "$key = '" . $value . "'";
          array_push($query_tok_values, $value);
        }
      }
    }

    // Add the where criteria.
    if (!empty($sql_criteria)) {
      $sql_criteria = implode(' AND ', $sql_criteria);
      $sql .= " WHERE $sql_criteria ";
    }

    // Add sorting on timestamp.
    $sql .= ' ORDER BY timestamp DESC';
    $result = pager_query($sql, $this->limit, 0, NULL, $query_tok_values);
    $matches = array();
    while ($row = db_fetch_array($result)) {
      $matches[] = $row;
    }

    // Return the result sets and matches.
    $result_sets = ceil(intval($this->searchCountQuery($sql)) / $this->limit);
    return (object) array(
      'result_sets' => $result_sets,
      'pagination' => theme('pager', array('quantity' => $this->quantity)),
      'matches' => $matches,
    );
  }

  /**
   * Function to get watchdog log detail.
   *
   * @param int $log_id
   *   The watchdog log id.
   *
   * @return null|object
   *   Object containing detailed watchdog log data.
   */
  public function getLogDetail($log_id) {
    $sql = "SELECT * FROM {watchdog} WHERE wid = %d";
    $result = db_query($sql, array($log_id));
    if ($row = db_fetch_array($result)) {
      return $row;
    }
    else {
      return NULL;
    }
  }

  /**
   * Function to get the logged log types in watchdog.
   *
   * @return array
   *   An array of watchdog log types.
   */
  public static function getLogTypes() {
    // Get the types in watchdog table.
    $log_types = array();
    $sql = "SELECT distinct type FROM {watchdog}";
    $result = db_query($sql);
    while ($row = db_fetch_object($result)) {
      $log_types[$row->type] = ucwords($row->type);
    }

    return $log_types;
  }
}
